<?php

$newBasePaymentHandler = dirname(__FILE__, 3) . '/handlers/mspaymenthandler.class.php';
$oldBasePaymentHandler = dirname(__FILE__, 3) . '/model/minishop2/mspaymenthandler.class.php';

if (!class_exists('msPaymentInterface')) {
    if (file_exists($newBasePaymentHandler)) {
        require_once $newBasePaymentHandler;
    } else {
        require_once $oldBasePaymentHandler;
    }
}


class VtbPay extends msPaymentHandler implements msPaymentInterface
{
    /**
     * VtbApi
     */

    /**
     * VtbApi settings and endpoints.
     * 
     * @var string|null
     */
    private $tokenUrl;
    
    /**
     * URL for creating orders in the payment system.
     *
     * @var string|null
     */
    private $orderUrl;
    
    /**
     * Mode of the payment system (test or production).
     *
     * @var string|null
     */
    private $mode;
    
    /**
     * Merchant authorization data for the client.
     *
     * @var mixed
     */
    private $clientMerchant;
    
    /**
     * Data for the client, including the grant type for OAuth.
     *
     * @var array
     */
    private $clientData = [
        'grant_type' => 'client_credentials'
    ];
    
    /**
     * Mapping of modes to their corresponding URLs.
     *
     * @var array
     */
    static private $modeUrls = [
        'PROD' => [
            'TOKEN' => 'https://open.api.vtb.ru:443/passport/oauth2/token',
            'ORDER' => 'https://gw.api.vtb.ru/openapi/smb/efcp/e-commerce/v1/orders'
        ],
        'TEST' => [
            'TOKEN' => 'https://auth.bankingapi.ru/auth/realms/kubernetes/protocol/openid-connect/token',
            'ORDER' => 'https://hackaton.bankingapi.ru/api/smb/efcp/e-commerce/api/v1/orders'
        ]
    ];
    
    
    /**
     * VtbPay constructor.
     *
     * This method initiates the VtbPay object by merging the default configuration with the user defined configurations.
     *
     * @param xPDOObject $object The xPDOObject from the modx system.
     * @param array $config User defined configuration array.
     */
    public function __construct(xPDOObject $object, $config = [])
    {
        parent::__construct($object, $config);
        
        $configPrefix = 'ms2_payment_vtbpay_';

        $this->config = array_merge([
            'client_id' => $this->modx->getOption($configPrefix . 'client_id'),
            'client_secret' => $this->modx->getOption($configPrefix . 'client_secret'),
            'merchant_authorization' => $this->modx->getOption($configPrefix . 'merchant_authorization'),
            'test_mode' => $this->modx->getOption($configPrefix . 'test_mode'),
            'logging' => $this->modx->getOption($configPrefix . 'logging'),
            'success_id' => $this->modx->getOption($configPrefix . 'success_id'),
            'failure_id' => $this->modx->getOption($configPrefix . 'failure_id')
        ], $config);
    }
    
    
    /**
     * Send method.
     *
     * This method is used to create a new order and send it to the payment gateway.
     * It then logs the response and returns the payment URL to the user.
     *
     * @param msOrder $order An instance of the order that needs to be sent to the gateway.
     * @return array|string Either the URL of the payment gateway or an error message.
     */
    public function send(msOrder $order)
    {
        $orderId = $order->get('id');
        
        try {
            $statusNew = $this->modx->getOption('ms2_status_new', null, 1);
            if ($order->get('status') > $statusNew) {
                throw new \Exception('ms2_err_status_wrong');
            }
            
            $profile = $order->getOne('UserProfile');
            $customerEmail = $profile->get('email');
            $cost = str_replace(',', '.', $order->get('cost'));
            
            // Get return url
            $siteUrl = $this->modx->getOption('site_url');
            $assetsUrl = $this->modx->getOption('assets_url') . 'components/minishop2/';
            $returnUrl = $siteUrl . substr($assetsUrl, 1) . 'payment/vtbpay.php';
            
            $response = $this->setRequestData()->getOrderLink(
                $orderId,
                $customerEmail,
                time(),
                $cost,
                $returnUrl
            );
            
            $this->vtbpayLogger(
                __FUNCTION__ . ' > getOrderLink', 
                modX::LOG_LEVEL_INFO, [
                    'arguments' => [
                        'orderId' => $orderId, 
                        'customerEmail' => $customerEmail,
                        'time' => time(), 
                        'cost' => $cost, 
                        'returnUrl' => $returnUrl
                    ],
                    'response' => $response
            ]);
            
            if (
                !isset($response['object']['payUrl']) ||
                empty($payUrl = $response['object']['payUrl'])
            ) {
                throw new \Exception('ms2_payment_vtbpay_url_fetch_error');
            }
            
            $this->setOrderProperty($order, 'payment_link', $payUrl);
            
            return $this->success('', [
                'redirect' => $payUrl,
                'msorder' => $orderId
            ]);
                
        } catch (\Exception $e) {
            // Handle exception and log error
            $this->vtbpayLogger(sprintf(
                __FUNCTION__ . ' > VtbPay Exception: %s; Order id: %s', 
                $this->modx->lexicon($e->getMessage()), 
                $orderId ?: ''
            ), modX::LOG_LEVEL_ERROR);
            
            // Set parameters and context
            $params['msorder'] = (int)$orderId;
            $context = $order->get('context');
            
            // Get failure url page
            if ($id = $this->modx->getOption('ms2_payment_vtbpay_failure_id', null, 0)) {
                $failure = $this->modx->makeUrl($id, $context, $params, 'full');
            }
            
            return $this->error($e->getMessage(), [
                'redirect' => $failure
            ]);
        }
    }
    
    
    /**
     * Get order property.
     *
     * This method is used to retrieve a specific property of an order.
     *
     * @param msOrder    $order The order for which the property needs to be fetched.
     * @param string     $name Name of the property to be fetched.
     * @param mixed|null $default The default value to return if the property doesn't exist.
     * @return mixed|null The property value if found, else the default value.
     */
    private function getOrderProperty(msOrder $order, $name, $default = null)
    {
        $props = $order->get('properties');

        return isset($props['payments']['vtbpay'][$name]) ? $props['payments']['vtbpay'][$name] : $default;
    }
    

    /**
     * Set order property.
     *
     * This method is used to set a property for an order. 
     * It handles both single and multiple property setting.
     *
     * @param msOrder      $order The order for which the property needs to be set.
     * @param string|array $name The name of the property or an array of properties to be set.
     * @param mixed|null   $value The value to be set for the property.
     */
    private function setOrderProperty(msOrder $order, $name, $value = null)
    {
        $newProperties = [];
        if (is_array($name)) {
            $newProperties = $name;
        } else {
            $newProperties[$name] = $value;
        }

        $orderProperties = $order->get('properties');
        
        if (isset($orderProperties['payments']['vtbpay'])) {
            $orderProperties['payments']['vtbpay'] = array_merge(
                $orderProperties['payments']['vtbpay'],
                $newProperties
            );
        }
        else {
            if (!is_array($orderProperties)) {
                $orderProperties = [];
            }
            $orderProperties['payments']['vtbpay'] = $newProperties;
        }
        
        $order->set('properties', $orderProperties);
        $order->save();
    }
    
    
    /**
     * Get payment link.
     *
     * This method is used to retrieve the payment link for an order.
     * Returns a direct link for continue payment process of existing order
     *
     * @param msOrder $order The order for which the payment link needs to be fetched.
     * @return string The URL of the payment gateway.
     */
    public function getPaymentLink(msOrder $order)
    {
        return $this->getOrderProperty($order, 'payment_link');
    }
    
    
    /**
     * Receive method.
     *
     * This method is used to receive the status of a payment from the payment gateway.
     * Depending on the response, it changes the status of the order accordingly.
     *
     * @param msOrder $order The order for which the payment status needs to be fetched.
     * @param array $params Any additional parameters needed.
     * @return bool True if the payment status is 'PAID', False otherwise.
     */
    public function receive(msOrder $order)
    {
        $orderId = $order->get('id');
        
        try {
            $response = $this->setRequestData()->getOrderInfo($orderId);
            
            $this->vtbpayLogger(
                __FUNCTION__ . ' > getOrderInfo', 
                modX::LOG_LEVEL_INFO, [
                    'arguments' => [
                        'orderId' => $orderId
                    ],
                    'response' => $response
            ]);
            
            if (
                !isset($response['object']['status']['value']) || 
                $response['object']['status']['value'] !== 'PAID'
            ) {
                throw new \Exception('Payment status is not PAID.');
            }
            
            $statusPaid = $this->modx->getOption('ms2_status_paid', null, 2);
            $this->ms2->changeOrderStatus($orderId, $statusPaid);
            return true;
            
        } catch (\Exception $e) {
            // Handle exception and log error
            $this->vtbpayLogger(sprintf(
                __FUNCTION__ . ' > VtbPay Exception: %s; Order id: %s', 
                $this->modx->lexicon($e->getMessage()), 
                $orderId ?: ''
            ), modX::LOG_LEVEL_ERROR);
            
            // Set status "cancelled"
            $statusCanceled = $this->modx->getOption('ms2_status_canceled', null, 4);
            $this->ms2->changeOrderStatus($orderId, $statusCanceled); 
            
            return false;
        }
    }
    
    
    
    /**
     * VtbApi
     */
     
    /**
     * Sets up the necessary request data for the VTB payment API
     *
     * @return $this Returns the current instance for method chaining
     */
    private function setRequestData() 
    {
        $this->clientData['client_id'] = trim($this->config['client_id']);
        $this->clientData['client_secret'] = trim($this->config['client_secret']);
        $this->clientMerchant = trim($this->config['merchant_authorization']);
        $this->mode = ($this->config['test_mode'] == 1) ? 'TEST': 'PROD';

        $this->tokenUrl = self::$modeUrls[$this->mode]['TOKEN'];
        $this->orderUrl = self::$modeUrls[$this->mode]['ORDER'];
        
        return $this;
    }
    

    /**
     * Retrieves the access token for the VTB payment API
     *
     * @return mixed The response from the API, which should include the access token
     */
    private function getAccessToken() 
    {
        return $this->sendRequest(
            $this->tokenUrl,
            http_build_query($this->clientData),
            [
                'Content-Type: application/x-www-form-urlencoded',
                'Merchant-Authorization: ' . $this->clientMerchant
            ]
        );
    }
    

    /**
     * Creates an order link for the specified parameters
     *
     * @param string $orderId The ID of the order
     * @param string $email The email of the customer
     * @param int $orderCreate The creation timestamp of the order
     * @param float $amount The amount of the order
     * @param string $returnUrl The URL to redirect to after the order
     * @return mixed The result from the sendRequest method, or false if the headers or merchant data is missing
     */
    private function getOrderLink(
        string $orderId,
        string $email,
        int $orderCreate,
        float $amount,
        string $returnUrl
    ) {
        $result = false;
        $headers = $this->getAutorizationHeaders();
        $merchant = $this->clientMerchant;

        if ($headers) {
            $headers[] = 'Content-Type: application/json';
            if ($merchant) {
                $headers[] = 'Merchant-Authorization: ' . $merchant;
            }

            $timestamp = $orderCreate + 86400; // +1d
            $expire = date('Y-m-d\TH:i:s.v\Z', $timestamp); // ISO 8601

            $args = [
                'orderId' => $orderId,
                'orderName' => 'Заказ #' . $orderId,
                'customer' => [
                    'email' => $email
                ],
                'amount' => [
                    'value' => $amount,
                    'code' => 'RUB'
                ],
                'expire' => $expire,
                'returnUrl' => $returnUrl
            ];

            $result = $this->sendRequest(
                $this->orderUrl,
                $args,
                $headers
            );
        }

        return $result;
    }
    

    /**
     * Retrieves the information for a specific order
     *
     * @param string $orderId The ID of the order
     * @return mixed The result from the sendRequest method, or false if the headers or merchant data is missing
     */
    private function getOrderInfo(string $orderId)
    {
        $result = false;
        $headers = $this->getAutorizationHeaders();
        $merchant = $this->clientMerchant;

        if ($headers) {
            if ($merchant) {
                $headers[] = 'Merchant-Authorization: ' . $merchant;
            }

            $result = $this->sendRequest(
                $this->orderUrl . '/' . $orderId,
                [],
                $headers,
                'GET'
            );
        }

        return $result;
    }
    

    /**
     * Generates the authorization headers for the API request
     *
     * @return array|bool The authorization headers if the access token exists, otherwise false
     */
    private function getAutorizationHeaders()
    {
        $token = $this->getAccessToken();
        
        if(!isset($token['access_token'])) {
          
            $errorDescription = '';
            if (
                isset($token['error_description']) && 
                !empty($token['error_description'])
            ) $errorDescription .= ' Error description: ' . $token['error_description'];
            
            throw new \Exception('Error getting access token.' . $errorDescription);
        }

        return [
            'X-IBM-Client-Id: ' . explode('@', $this->clientData['client_id'])[0],
            'Authorization: ' . $token['token_type'] . ' ' . $token['access_token'],
        ];
    }
    
    
    /**
     * Sends a HTTP request to the provided URL
     *
     * @param string $url The URL to send the request to
     * @param array $data The data to send with the request
     * @param array $headers The headers to send with the request
     * @param string $method The HTTP method to use for the request (default 'POST')
     * @param bool $json_encode Whether or not to JSON encode the data (default false)
     * @return mixed The result of the request, decoded from JSON, or false if there was an error with the request
     */
    private function sendRequest(
        $url,
        $data = [],
        $headers = [],
        $method = 'POST',
        $jsonEncode = true
    ) {
        $curl = curl_init();
        
        if (in_array('Content-Type: application/json', $headers) && $jsonEncode) {
            $data = json_encode($data);
        }

        if ($method === 'POST') {
            curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        } 
        elseif ($method === 'PUT') {
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'PUT');
            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        } 
        else {
            $url .= is_array($data) ? '?' . http_build_query($data) : '';
        }

        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

        $result = curl_exec($curl);
        
        if ($result === false) {
            throw new \Exception('Curl error: ' . curl_error($curl));
        }
        
        if (empty($result)) {
            throw new \Exception('The curl request returned an empty response');
        }
        
        $result = json_decode($result, true);
        
        if (json_last_error() != JSON_ERROR_NONE) {
            throw new \Exception('Json decode error: ' . json_last_error_msg());
        }

        curl_close($curl);

        return $result;
    }
    
  
    /**
     * VtbPay logger.
     *
     * This method is used for logging messages. 
     * Depending on the settings, it either prints the message to the log or simply returns false.
     *
     * @param string $message The message to be logged.
     * @param mixed $type The type of the log message.
     * @param array $context Additional context information for the log.
     * @param bool $print_r Flag to determine whether to use print_r or json_encode for the context.
     * @return bool False if logging is not enabled, no return value otherwise.
     */
    public function vtbpayLogger(
        $message, 
        $type = '',
        $context = [],
        $print_r = false
    ) {
        if (empty($type)) $type = modX::LOG_LEVEL_INFO;
        
        if (
            empty($message) || 
            $type !== modX::LOG_LEVEL_ERROR &&
            (
              !isset($this->config['logging']) || 
              $this->config['logging'] !== '1'
            )
        ) return null;
        
        $message = '[miniShop2:VtbPay] ' . $message;
        
        if (!empty($context) && is_array($context)) {
            $message .= ' Context: '. PHP_EOL;
            if ($print_r) {
                $message .= print_r($context, true);
            }
            else {
                $message .= json_encode($context);
            }
        }
        
        $this->modx->setLogLevel($type);
        
        $this->modx->setLogTarget([
           'target' => 'FILE',
           'options' => [
               'filename' => 'vtbpay.log'
            ]
        ]);

        $this->modx->log(
            $type,
            $message
        );
    }
}